/* Copyright 2021 Enjin Pte. Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "gtest/gtest.h"
#include "JsonTestSuite.hpp"
#include "enjinsdk/models/Player.hpp"
#include <string>

using namespace enjin::sdk::models;
using namespace enjin::test::suites;

class PlayerTest : public JsonTestSuite,
                   public testing::Test {
public:
    Player class_under_test;

    constexpr static char POPULATED_JSON_OBJECT[] =
            R"({"id":"1","linkingInfo":{},"wallet":{},"createdAt":"1","updatedAt":"1"})";
};

TEST_F(PlayerTest, DeserializeEmptyStringFieldsDoNotHaveValues) {
    // Arrange
    const std::string json;

    // Act
    class_under_test.deserialize(json);

    // Assert
    EXPECT_FALSE(class_under_test.get_id().has_value());
    EXPECT_FALSE(class_under_test.get_linking_info().has_value());
    EXPECT_FALSE(class_under_test.get_wallet().has_value());
    EXPECT_FALSE(class_under_test.get_created_at().has_value());
    EXPECT_FALSE(class_under_test.get_updated_at().has_value());
}

TEST_F(PlayerTest, DeserializeEmptyJsonObjectFieldsDoNotHaveValues) {
    // Arrange
    const std::string json(EMPTY_JSON_OBJECT);

    // Act
    class_under_test.deserialize(json);

    // Assert
    EXPECT_FALSE(class_under_test.get_id().has_value());
    EXPECT_FALSE(class_under_test.get_linking_info().has_value());
    EXPECT_FALSE(class_under_test.get_wallet().has_value());
    EXPECT_FALSE(class_under_test.get_created_at().has_value());
    EXPECT_FALSE(class_under_test.get_updated_at().has_value());
}

TEST_F(PlayerTest, DeserializePopulatedJsonObjectFieldsHaveExpectedValues) {
    // Arrange
    const std::string expected_string("1");
    const LinkingInfo expected_linking_info;
    const Wallet expected_wallet;
    const std::string json(POPULATED_JSON_OBJECT);

    // Act
    class_under_test.deserialize(json);

    // Assert
    EXPECT_EQ(expected_string, class_under_test.get_id().value());
    EXPECT_EQ(expected_linking_info, class_under_test.get_linking_info().value());
    EXPECT_EQ(expected_wallet, class_under_test.get_wallet().value());
    EXPECT_EQ(expected_string, class_under_test.get_created_at().value());
    EXPECT_EQ(expected_string, class_under_test.get_updated_at().value());
}

TEST_F(PlayerTest, EqualityNeitherSideIsPopulatedReturnsTrue) {
    // Arrange
    Player lhs;
    Player rhs;

    // Act
    bool actual = lhs == rhs;

    // Assert
    ASSERT_TRUE(actual);
}

TEST_F(PlayerTest, EqualityBothSidesArePopulatedReturnsTrue) {
    // Arrange
    Player lhs;
    Player rhs;
    lhs.deserialize(POPULATED_JSON_OBJECT);
    rhs.deserialize(POPULATED_JSON_OBJECT);

    // Act
    bool actual = lhs == rhs;

    // Assert
    ASSERT_TRUE(actual);
}

TEST_F(PlayerTest, EqualityLeftSideIsPopulatedReturnsFalse) {
    // Arrange
    Player lhs;
    Player rhs;
    lhs.deserialize(POPULATED_JSON_OBJECT);

    // Act
    bool actual = lhs == rhs;

    // Assert
    ASSERT_FALSE(actual);
}

TEST_F(PlayerTest, EqualityRightSideIsPopulatedReturnsFalse) {
    // Arrange
    Player lhs;
    Player rhs;
    rhs.deserialize(POPULATED_JSON_OBJECT);

    // Act
    bool actual = lhs == rhs;

    // Assert
    ASSERT_FALSE(actual);
}

TEST_F(PlayerTest, CopyOperatorCopyEqualsOriginal) {
    // Arrange
    Player original;
    Player copy;
    original.deserialize(POPULATED_JSON_OBJECT);

    // Act
    copy = original;

    // Assert
    ASSERT_EQ(original, copy);
}
